@@ -1,28 +1,25 @@
 """
 Uses Tensorflow to create a generic DNN which learns on all of the features in a
-repo.
+record.
 """
 import os
 import abc
-import pydoc
 import hashlib
 import inspect
-from typing import List, Dict, Any, AsyncIterator, Tuple, Optional, Type
+from typing import List, Dict, Any, AsyncIterator, Type
 
 import numpy as np
-import tensorflow
 
-from dffml.repo import Repo
-from dffml.feature import Feature, Features
-from dffml.source.source import Sources
-from dffml.model.model import ModelConfig, ModelContext, Model, ModelNotTrained
-from dffml.accuracy import Accuracy
-from dffml.util.entrypoint import entry_point
-from dffml.base import BaseConfig
-from dffml.util.cli.arg import Arg
-from dffml.feature.feature import Feature, Features
-from dffml.util.cli.parser import list_action
+os.environ["TF_CPP_MIN_LOG_LEVEL"] = "2"
+import tensorflow as tf
+
+from dffml.record import Record
+from dffml.model.accuracy import Accuracy
 from dffml.base import config, field
+from dffml.source.source import Sources
+from dffml.util.entrypoint import entrypoint
+from dffml.feature.feature import Feature, Features
+from dffml.model.model import ModelContext, Model, ModelNotTrained
 
 
 class TensorflowModelContext(ModelContext):
@@ -40,7 +37,7 @@ class TensorflowModelContext(ModelContext):
 
     def _feature_columns(self):
         """
-        Converts repos into training data
+        Converts records into training data
         """
         cols: Dict[str, Any] = {}
         for feature in self.parent.config.features:
@@ -65,7 +62,7 @@ class TensorflowModelContext(ModelContext):
             or dtype is float
             or issubclass(dtype, float)
         ):
-            return tensorflow.feature_column.numeric_column(
+            return tf.feature_column.numeric_column(
                 feature.NAME, shape=feature.length()
             )
         self.logger.warning(
@@ -95,38 +92,33 @@ class TensorflowModelContext(ModelContext):
             )
         return os.path.join(self.parent.config.directory, model)
 
-    async def predict_input_fn(self, repos: AsyncIterator[Repo], **kwargs):
+    async def predict_input_fn(self, records: AsyncIterator[Record], **kwargs):
         """
-        Uses the numpy input function with data from repo features.
+        Uses the numpy input function with data from record features.
         """
         x_cols: Dict[str, Any] = {feature: [] for feature in self.features}
-        ret_repos = []
-        async for repo in repos:
-            if not repo.features(self.features):
+        ret_records = []
+        async for record in records:
+            if not record.features(self.features):
                 continue
-            ret_repos.append(repo)
-            for feature, results in repo.features(self.features).items():
+            ret_records.append(record)
+            for feature, results in record.features(self.features).items():
                 x_cols[feature].append(np.array(results))
         for feature in x_cols:
             x_cols[feature] = np.array(x_cols[feature])
-        self.logger.info("------ Repo Data ------")
+        self.logger.info("------ Record Data ------")
         self.logger.info("x_cols:    %d", len(list(x_cols.values())[0]))
         self.logger.info("-----------------------")
-        input_fn = tensorflow.estimator.inputs.numpy_input_fn(
+        input_fn = tf.compat.v1.estimator.inputs.numpy_input_fn(
             x_cols, shuffle=False, num_epochs=1, **kwargs
         )
-        return input_fn, ret_repos
+        return input_fn, ret_records
 
     async def train(self, sources: Sources):
         """
         Train on data submitted via classify.
         """
-        input_fn = await self.training_input_fn(
-            sources,
-            batch_size=20,
-            shuffle=True,
-            epochs=self.parent.config.epochs,
-        )
+        input_fn = await self.training_input_fn(sources)
         self.model.train(input_fn=input_fn, steps=self.parent.config.steps)
 
     @property
@@ -139,13 +131,19 @@ class TensorflowModelContext(ModelContext):
 
 @config
 class DNNClassifierModelConfig:
-    classification: str = field("Feature name holding classification value")
+    predict: Feature = field("Feature name holding predict value")
     classifications: List[str] = field("Options for value of classification")
     features: Features = field("Features to train on")
     clstype: Type = field("Data type of classifications values", default=str)
+    batchsize: int = field(
+        "Number records to pass through in an epoch", default=20
+    )
+    shuffle: bool = field(
+        "Randomise order of records in a batch", default=True
+    )
     steps: int = field("Number of steps to train the model", default=3000)
     epochs: int = field(
-        "Number of iterations to pass over all repos in a source", default=30
+        "Number of iterations to pass over all records in a source", default=30
     )
     directory: str = field(
         "Directory where state should be saved",
@@ -174,13 +172,9 @@ class DNNClassifierModelContext(TensorflowModelContext):
         self.classifications = self._classifications(self.cids)
         self.model_dir_path = self._model_dir_path()
 
-    @property
-    def classification(self):
-        return self.parent.config.classification
-
     def _mkcids(self, classifications):
         """
-        Create an index, possible classification mapping and sort the list of
+        Create an index, possible predict mapping and sort the list of
         classifications first.
         """
         cids = dict(
@@ -211,7 +205,7 @@ class DNNClassifierModelContext(TensorflowModelContext):
             len(self.classifications),
             self.classifications,
         )
-        self._model = tensorflow.estimator.DNNClassifier(
+        self._model = tf.compat.v1.estimator.DNNClassifier(
             feature_columns=list(self.feature_columns.values()),
             hidden_units=self.parent.config.hidden,
             n_classes=len(self.parent.config.classifications),
@@ -219,124 +213,117 @@ class DNNClassifierModelContext(TensorflowModelContext):
         )
         return self._model
 
-    async def training_input_fn(
-        self,
-        sources: Sources,
-        batch_size=20,
-        shuffle=False,
-        epochs=1,
-        **kwargs,
-    ):
+    async def training_input_fn(self, sources: Sources, **kwargs):
         """
-        Uses the numpy input function with data from repo features.
+        Uses the numpy input function with data from record features.
         """
         self.logger.debug("Training on features: %r", self.features)
         x_cols: Dict[str, Any] = {feature: [] for feature in self.features}
         y_cols = []
-        for repo in [
-            repo
-            async for repo in sources.with_features(
-                self.features + [self.classification]
+        for record in [
+            record
+            async for record in sources.with_features(
+                self.features + [self.parent.config.predict.NAME]
             )
-            if repo.feature(self.classification) in self.classifications
+            if record.feature(self.parent.config.predict.NAME)
+            in self.classifications
         ]:
-            for feature, results in repo.features(self.features).items():
+            for feature, results in record.features(self.features).items():
                 x_cols[feature].append(np.array(results))
             y_cols.append(
-                self.classifications[repo.feature(self.classification)]
+                self.classifications[
+                    record.feature(self.parent.config.predict.NAME)
+                ]
             )
         if not y_cols:
-            raise ValueError("No repos to train on")
+            raise ValueError("No records to train on")
         y_cols = np.array(y_cols)
         for feature in x_cols:
             x_cols[feature] = np.array(x_cols[feature])
-        self.logger.info("------ Repo Data ------")
+        self.logger.info("------ Record Data ------")
         self.logger.info("x_cols:    %d", len(list(x_cols.values())[0]))
         self.logger.info("y_cols:    %d", len(y_cols))
         self.logger.info("-----------------------")
-        input_fn = tensorflow.estimator.inputs.numpy_input_fn(
+        input_fn = tf.compat.v1.estimator.inputs.numpy_input_fn(
             x_cols,
             y_cols,
-            batch_size=batch_size,
-            shuffle=shuffle,
-            num_epochs=epochs,
+            batch_size=self.parent.config.batchsize,
+            shuffle=self.parent.config.shuffle,
+            num_epochs=self.parent.config.epochs,
             **kwargs,
         )
         return input_fn
 
-    async def accuracy_input_fn(
-        self,
-        sources: Sources,
-        batch_size=20,
-        shuffle=False,
-        epochs=1,
-        **kwargs,
-    ):
+    async def accuracy_input_fn(self, sources: Sources, **kwargs):
         """
-        Uses the numpy input function with data from repo features.
+        Uses the numpy input function with data from record features.
         """
         x_cols: Dict[str, Any] = {feature: [] for feature in self.features}
         y_cols = []
-        for repo in [
-            repo
-            async for repo in sources.with_features(
-                self.features + [self.classification]
+        for record in [
+            record
+            async for record in sources.with_features(
+                self.features + [self.parent.config.predict.NAME]
             )
-            if repo.feature(self.classification) in self.classifications
+            if record.feature(self.parent.config.predict.NAME)
+            in self.classifications
         ]:
-            for feature, results in repo.features(self.features).items():
+            for feature, results in record.features(self.features).items():
                 x_cols[feature].append(np.array(results))
             y_cols.append(
-                self.classifications[repo.feature(self.classification)]
+                self.classifications[
+                    record.feature(self.parent.config.predict.NAME)
+                ]
             )
         y_cols = np.array(y_cols)
         for feature in x_cols:
             x_cols[feature] = np.array(x_cols[feature])
-        self.logger.info("------ Repo Data ------")
+        self.logger.info("------ Record Data ------")
         self.logger.info("x_cols:    %d", len(list(x_cols.values())[0]))
         self.logger.info("y_cols:    %d", len(y_cols))
         self.logger.info("-----------------------")
-        input_fn = tensorflow.estimator.inputs.numpy_input_fn(
+        input_fn = tf.compat.v1.estimator.inputs.numpy_input_fn(
             x_cols,
             y_cols,
-            batch_size=batch_size,
-            shuffle=shuffle,
-            num_epochs=epochs,
+            batch_size=self.parent.config.batchsize,
+            shuffle=self.parent.config.shuffle,
+            num_epochs=1,
             **kwargs,
         )
         return input_fn
 
     async def accuracy(self, sources: Sources) -> Accuracy:
         """
-        Evaluates the accuracy of our model after training using the input repos
+        Evaluates the accuracy of our model after training using the input records
         as test data.
         """
         if not os.path.isdir(self.model_dir_path):
             raise ModelNotTrained("Train model before assessing for accuracy.")
-        input_fn = await self.accuracy_input_fn(
-            sources, batch_size=20, shuffle=False, epochs=1
-        )
+        input_fn = await self.accuracy_input_fn(sources)
         accuracy_score = self.model.evaluate(input_fn=input_fn)
         return Accuracy(accuracy_score["accuracy"])
 
-    async def predict(self, repos: AsyncIterator[Repo]) -> AsyncIterator[Repo]:
+    async def predict(
+        self, records: AsyncIterator[Record]
+    ) -> AsyncIterator[Record]:
         """
-        Uses trained data to make a prediction about the quality of a repo.
+        Uses trained data to make a prediction about the quality of a record.
         """
         if not os.path.isdir(self.model_dir_path):
             raise ModelNotTrained("Train model before prediction.")
         # Create the input function
-        input_fn, predict = await self.predict_input_fn(repos)
+        input_fn, predict = await self.predict_input_fn(records)
         # Makes predictions on classifications
         predictions = self.model.predict(input_fn=input_fn)
-        for repo, pred_dict in zip(predict, predictions):
+        target = self.parent.config.predict.NAME
+        for record, pred_dict in zip(predict, predictions):
             class_id = pred_dict["class_ids"][0]
             probability = pred_dict["probabilities"][class_id]
-            repo.predicted(self.cids[class_id], probability)
-            yield repo
+            record.predicted(target, self.cids[class_id], probability)
+            yield record
 
 
-@entry_point("tfdnnc")
+@entrypoint("tfdnnc")
 class DNNClassifierModel(Model):
     """
     Implemented using Tensorflow's DNNClassifier.
@@ -352,44 +339,44 @@ class DNNClassifierModel(Model):
             -model tfdnnc \\
             -model-epochs 3000 \\
             -model-steps 20000 \\
-            -model-classification classification \\
+            -model-predict classification:int:1 \\
             -model-classifications 0 1 2 \\
             -model-clstype int \\
             -sources iris=csv \\
             -source-filename iris_training.csv \\
             -model-features \\
-              def:SepalLength:float:1 \\
-              def:SepalWidth:float:1 \\
-              def:PetalLength:float:1 \\
-              def:PetalWidth:float:1 \\
+              SepalLength:float:1 \\
+              SepalWidth:float:1 \\
+              PetalLength:float:1 \\
+              PetalWidth:float:1 \\
             -log debug
         ... lots of output ...
         $ dffml accuracy \\
             -model tfdnnc \\
-            -model-classification classification \\
+            -model-predict classification:int:1 \\
             -model-classifications 0 1 2 \\
             -model-clstype int \\
             -sources iris=csv \\
             -source-filename iris_test.csv \\
             -model-features \\
-              def:SepalLength:float:1 \\
-              def:SepalWidth:float:1 \\
-              def:PetalLength:float:1 \\
-              def:PetalWidth:float:1 \\
+              SepalLength:float:1 \\
+              SepalWidth:float:1 \\
+              PetalLength:float:1 \\
+              PetalWidth:float:1 \\
             -log critical
         0.99996233782
         $ dffml predict all \\
             -model tfdnnc \\
-            -model-classification classification \\
+            -model-predict classification:int:1 \\
             -model-classifications 0 1 2 \\
             -model-clstype int \\
             -sources iris=csv \\
             -source-filename iris_test.csv \\
             -model-features \\
-              def:SepalLength:float:1 \\
-              def:SepalWidth:float:1 \\
-              def:PetalLength:float:1 \\
-              def:PetalWidth:float:1 \\
+              SepalLength:float:1 \\
+              SepalWidth:float:1 \\
+              PetalLength:float:1 \\
+              PetalWidth:float:1 \\
             -caching \\
             -log critical \\
           > results.json
@@ -406,10 +393,13 @@ class DNNClassifierModel(Model):
                 },
                 "last_updated": "2019-07-31T02:00:12Z",
                 "prediction": {
-                    "confidence": 0.9999997615814209,
-                    "value": 1
+                    "classification":
+                        {
+                            "confidence": 0.9999997615814209,
+                            "value": 1
+                        }
                 },
-                "src_url": "0"
+                "key": "0"
             },
             {
                 "extra": {},
@@ -422,10 +412,13 @@ class DNNClassifierModel(Model):
                 },
                 "last_updated": "2019-07-31T02:00:12Z",
                 "prediction": {
-                    "confidence": 0.9999984502792358,
-                    "value": 2
+                    "classification":
+                    {
+                        "confidence": 0.9999984502792358,
+                        "value": 2
+                    }
                 },
-                "src_url": "1"
+                "key": "1"
             },
 
     """
