@@ -1,33 +1,33 @@
 """
 Uses Tensorflow to create a generic DNN which learns on all of the features in a
-repo.
+record.
 """
 import os
 from typing import List, Dict, Any, AsyncIterator
 
 import numpy as np
-import tensorflow
 
-from dffml.repo import Repo
-from dffml.source.source import Sources
+os.environ["TF_CPP_MIN_LOG_LEVEL"] = "2"
+import tensorflow as tf
+
+from dffml.record import Record
 from dffml.model.model import Model
-from dffml.accuracy import Accuracy
-from dffml.util.entrypoint import entry_point
-from dffml.base import BaseConfig, config, field
-from dffml.util.cli.arg import Arg
+from dffml.model.accuracy import Accuracy
+from dffml.source.source import Sources
+from dffml.util.entrypoint import entrypoint
+from dffml.base import config, field
 from dffml.feature.feature import Feature, Features
-from dffml.util.cli.parser import list_action
 
-from dffml_model_tensorflow.dnnc import TensorflowModelContext
+from .dnnc import TensorflowModelContext
 
 
 @config
 class DNNRegressionModelConfig:
-    predict: str = field("Feature name holding target values")
+    predict: Feature = field("Feature name holding target values")
     features: Features = field("Features to train on")
     steps: int = field("Number of steps to train the model", default=3000)
     epochs: int = field(
-        "Number of iterations to pass over all repos in a source", default=30
+        "Number of iterations to pass over all records in a source", default=30
     )
     directory: str = field(
         "Directory where state should be saved",
@@ -51,7 +51,7 @@ class DNNRegressionModelContext(TensorflowModelContext):
         super().__init__(parent)
         self.model_dir_path = self._model_dir_path()
         self.all_features = self.parent.config.features.names() + [
-            self.parent.config.predict
+            self.parent.config.predict.NAME
         ]
         self.features = self._applicable_features()
 
@@ -64,9 +64,7 @@ class DNNRegressionModelContext(TensorflowModelContext):
             return self._model
         self.logger.debug("Loading model ")
 
-        _head = tensorflow.contrib.estimator.regression_head()
-        self._model = tensorflow.estimator.DNNEstimator(
-            head=_head,
+        self._model = tf.compat.v1.estimator.DNNRegressor(
             feature_columns=list(self.feature_columns.values()),
             hidden_units=self.parent.config.hidden,
             model_dir=self.model_dir_path,
@@ -83,26 +81,26 @@ class DNNRegressionModelContext(TensorflowModelContext):
         **kwargs,
     ):
         """
-        Uses the numpy input function with data from repo features.
+        Uses the numpy input function with data from record features.
         """
         self.logger.debug("Training on features: %r", self.features)
         x_cols: Dict[str, Any] = {feature: [] for feature in self.features}
         y_cols = []
 
-        async for repo in sources.with_features(self.all_features):
-            for feature, results in repo.features(self.features).items():
+        async for record in sources.with_features(self.all_features):
+            for feature, results in record.features(self.features).items():
 
                 x_cols[feature].append(np.array(results))
-            y_cols.append(repo.feature(self.parent.config.predict))
+            y_cols.append(record.feature(self.parent.config.predict.NAME))
 
         y_cols = np.array(y_cols)
         for feature in x_cols:
             x_cols[feature] = np.array(x_cols[feature])
-        self.logger.info("------ Repo Data ------")
+        self.logger.info("------ Record Data ------")
         self.logger.info("x_cols:    %d", len(list(x_cols.values())[0]))
         self.logger.info("y_cols:    %d", len(y_cols))
         self.logger.info("-----------------------")
-        input_fn = tensorflow.estimator.inputs.numpy_input_fn(
+        input_fn = tf.compat.v1.estimator.inputs.numpy_input_fn(
             x_cols,
             y_cols,
             batch_size=batch_size,
@@ -121,24 +119,24 @@ class DNNRegressionModelContext(TensorflowModelContext):
         **kwargs,
     ):
         """
-        Uses the numpy input function with data from repo features.
+        Uses the numpy input function with data from record features.
         """
         x_cols: Dict[str, Any] = {feature: [] for feature in self.features}
         y_cols = []
 
-        async for repo in sources.with_features(self.all_features):
-            for feature, results in repo.features(self.features).items():
+        async for record in sources.with_features(self.all_features):
+            for feature, results in record.features(self.features).items():
                 x_cols[feature].append(np.array(results))
-            y_cols.append(repo.feature(self.parent.config.predict))
+            y_cols.append(record.feature(self.parent.config.predict.NAME))
 
         y_cols = np.array(y_cols)
         for feature in x_cols:
             x_cols[feature] = np.array(x_cols[feature])
-        self.logger.info("------ Repo Data ------")
+        self.logger.info("------ Record Data ------")
         self.logger.info("x_cols:    %d", len(list(x_cols.values())[0]))
         self.logger.info("y_cols:    %d", len(y_cols))
         self.logger.info("-----------------------")
-        input_fn = tensorflow.estimator.inputs.numpy_input_fn(
+        input_fn = tf.compat.v1.estimator.inputs.numpy_input_fn(
             x_cols,
             y_cols,
             batch_size=batch_size,
@@ -150,7 +148,7 @@ class DNNRegressionModelContext(TensorflowModelContext):
 
     async def accuracy(self, sources: Sources) -> Accuracy:
         """
-        Evaluates the accuracy of our model after training using the input repos
+        Evaluates the accuracy of our model after training using the input records
         as test data.
         """
         if not os.path.isdir(self.model_dir_path):
@@ -161,26 +159,29 @@ class DNNRegressionModelContext(TensorflowModelContext):
         metrics = self.model.evaluate(input_fn=input_fn)
         return Accuracy(1 - metrics["loss"])  # 1 - mse
 
-    async def predict(self, repos: AsyncIterator[Repo]) -> AsyncIterator[Repo]:
+    async def predict(
+        self, records: AsyncIterator[Record]
+    ) -> AsyncIterator[Record]:
         """
-        Uses trained data to make a prediction about the quality of a repo.
+        Uses trained data to make a prediction about the quality of a record.
         """
 
         if not os.path.isdir(self.model_dir_path):
             raise NotADirectoryError("Model not trained")
         # Create the input function
-        input_fn, predict_repo = await self.predict_input_fn(repos)
+        input_fn, predict_record = await self.predict_input_fn(records)
         # Makes predictions on
         predictions = self.model.predict(input_fn=input_fn)
-
-        for repo, pred_dict in zip(predict_repo, predictions):
+        target = self.parent.config.predict.NAME
+        for record, pred_dict in zip(predict_record, predictions):
             # TODO Instead of float("nan") save accuracy value and use that.
-            repo.predicted(float(pred_dict["predictions"]), float("nan"))
-
-            yield repo
+            record.predicted(
+                target, float(pred_dict["predictions"]), float("nan")
+            )
+            yield record
 
 
-@entry_point("tfdnnr")
+@entrypoint("tfdnnr")
 class DNNRegressionModel(Model):
     """
     Implemented using Tensorflow's DNNEstimator.
@@ -216,39 +217,36 @@ class DNNRegressionModel(Model):
             -model tfdnnr \\
             -model-epochs 300 \\
             -model-steps 2000 \\
-            -model-predict TARGET \\
+            -model-predict TARGET:float:1 \\
             -model-hidden 8 16 8 \\
             -sources s=csv \\
-            -source-readonly \\
             -source-filename train.csv \\
             -model-features \\
-              def:Feature1:float:1 \\
-              def:Feature2:float:1 \\
+              Feature1:float:1 \\
+              Feature2:float:1 \\
             -log debug
         Enabling debug log shows tensorflow losses...
         $ dffml accuracy \\
             -model tfdnnr \\
-            -model-predict TARGET \\
+            -model-predict TARGET:float:1 \\
             -model-hidden 8 16 8 \\
             -sources s=csv \\
-            -source-readonly \\
             -source-filename test.csv \\
             -model-features \\
-              def:Feature1:float:1 \\
-              def:Feature2:float:1 \\
+              Feature1:float:1 \\
+              Feature2:float:1 \\
             -log critical
         0.9468210011
         $ echo -e 'Feature1,Feature2,TARGET\\n0.21,0.18,0.84\\n' | \\
           dffml predict all \\
             -model tfdnnr \\
-            -model-predict TARGET \\
+            -model-predict TARGET:float:1 \\
             -model-hidden 8 16 8 \\
             -sources s=csv \\
-            -source-readonly \\
             -source-filename /dev/stdin \\
             -model-features \\
-              def:Feature1:float:1 \\
-              def:Feature2:float:1 \\
+              Feature1:float:1 \\
+              Feature2:float:1 \\
             -log critical
         [
             {
@@ -260,10 +258,12 @@ class DNNRegressionModel(Model):
                 },
                 "last_updated": "2019-10-24T15:26:41Z",
                 "prediction": {
-                    "confidence": NaN,
-                    "value": 1.1983429193496704
+                    "TARGET" : {
+                        "confidence": NaN,
+                        "value": 1.1983429193496704
+                    }
                 },
-                "src_url": 0
+                "key": 0
             }
         ]
 
