@@ -3,10 +3,10 @@ import random
 import tempfile
 from typing import Type
 
-from dffml.repo import Repo, RepoData
+from dffml.record import Record, RecordData
 from dffml.source.source import Sources
 from dffml.source.memory import MemorySource, MemorySourceConfig
-from dffml.feature import Data, Feature, Features
+from dffml.feature import Data, Feature, Features, DefFeature
 from dffml.util.cli.arg import parse_unknown
 from dffml.util.asynctestcase import AsyncTestCase
 
@@ -33,30 +33,30 @@ class TestDNN(AsyncTestCase):
         cls.model_dir = tempfile.TemporaryDirectory()
         cls.feature = StartsWithA()
         cls.features = Features(cls.feature)
-        cls.repos = [
-            Repo(
+        cls.records = [
+            Record(
                 "a" + str(random.random()),
                 data={"features": {cls.feature.NAME: 1, "string": "a"}},
             )
             for _ in range(0, 1000)
         ]
-        cls.repos += [
-            Repo(
+        cls.records += [
+            Record(
                 "b" + str(random.random()),
                 data={"features": {cls.feature.NAME: 0, "string": "not a"}},
             )
             for _ in range(0, 1000)
         ]
         cls.sources = Sources(
-            MemorySource(MemorySourceConfig(repos=cls.repos))
+            MemorySource(MemorySourceConfig(records=cls.records))
         )
         cls.model = DNNClassifierModel(
             DNNClassifierModelConfig(
                 directory=cls.model_dir.name,
                 steps=1000,
-                epochs=30,
-                hidden=[10, 20, 10],
-                classification="string",
+                epochs=40,
+                hidden=[50, 20, 10],
+                predict=DefFeature("string", str, 1),
                 classifications=["a", "not a"],
                 clstype=str,
                 features=cls.features,
@@ -70,8 +70,8 @@ class TestDNN(AsyncTestCase):
     async def test_config(self):
         config = self.model.__class__.config(
             parse_unknown(
-                "--model-classification",
-                "feature_name",
+                "--model-predict",
+                "feature_name:int:1",
                 "--model-classifications",
                 "0",
                 "1",
@@ -79,7 +79,7 @@ class TestDNN(AsyncTestCase):
                 "--model-clstype",
                 "int",
                 "--model-features",
-                "def:starts_with_a:int:1",
+                "starts_with_a:int:1",
             )
         )
         self.assertEqual(
@@ -91,7 +91,7 @@ class TestDNN(AsyncTestCase):
         self.assertEqual(config.steps, 3000)
         self.assertEqual(config.epochs, 30)
         self.assertEqual(config.hidden, [12, 40, 15])
-        self.assertEqual(config.classification, "feature_name")
+        self.assertEqual(config.predict.NAME, "feature_name")
         self.assertEqual(config.classifications, [0, 1, 2])
         self.assertEqual(config.clstype, int)
 
@@ -107,12 +107,13 @@ class TestDNN(AsyncTestCase):
                 self.assertGreater(res, 0.9)
 
     async def test_02_predict(self):
-        a = Repo("a", data={"features": {self.feature.NAME: 1}})
+        a = Record("a", data={"features": {self.feature.NAME: 1}})
         async with Sources(
-            MemorySource(MemorySourceConfig(repos=[a]))
+            MemorySource(MemorySourceConfig(records=[a]))
         ) as sources, self.model as model:
+            target_name = model.config.predict.NAME
             async with sources() as sctx, model() as mctx:
-                res = [repo async for repo in mctx.predict(sctx.repos())]
+                res = [record async for record in mctx.predict(sctx.records())]
                 self.assertEqual(len(res), 1)
-            self.assertEqual(res[0].src_url, a.src_url)
-            self.assertTrue(res[0].prediction().value)
+            self.assertEqual(res[0].key, a.key)
+            self.assertTrue(res[0].prediction(target_name).value)
