@@ -4,12 +4,13 @@ import tempfile
 from typing import Type
 
 import numpy as np
-from dffml.repo import Repo, RepoData
+
+from dffml.record import Record
 from dffml.source.source import Sources
 from dffml.source.memory import MemorySource, MemorySourceConfig
-from dffml.feature import Data, Feature, Features
 from dffml.util.cli.arg import parse_unknown
 from dffml.util.asynctestcase import AsyncTestCase
+from dffml.feature import Feature, Features, DefFeature
 
 from dffml_model_tensorflow.dnnr import (
     DNNRegressionModel,
@@ -50,17 +51,17 @@ class TestDNN(AsyncTestCase):
             DNNRegressionModelConfig(
                 directory=cls.model_dir.name,
                 steps=1000,
-                epochs=30,
-                hidden=[10, 20, 10],
-                predict="TARGET",
+                epochs=40,
+                hidden=[50, 20, 10],
+                predict=DefFeature("TARGET", float, 1),
                 features=cls.features,
             )
         )
         # Generating data f(x1,x2) = 2*x1 + 3*x2
-        _n_data = 1000
+        _n_data = 2000
         _temp_data = np.random.rand(2, _n_data)
-        cls.repos = [
-            Repo(
+        cls.records = [
+            Record(
                 "x" + str(random.random()),
                 data={
                     "features": {
@@ -73,7 +74,7 @@ class TestDNN(AsyncTestCase):
             for i in range(0, _n_data)
         ]
         cls.sources = Sources(
-            MemorySource(MemorySourceConfig(repos=cls.repos))
+            MemorySource(MemorySourceConfig(records=cls.records))
         )
 
     @classmethod
@@ -85,11 +86,11 @@ class TestDNN(AsyncTestCase):
         config = self.model.__class__.config(
             parse_unknown(
                 "--model-predict",
-                "TARGET",
+                "TARGET:float:1",
                 "--model-features",
-                "def:feature_1:float:1",
+                "feature_1:float:1",
                 "--model-features",
-                "def:feature_2:float:1",
+                "feature_2:float:1",
             )
         )
         self.assertEqual(
@@ -101,7 +102,7 @@ class TestDNN(AsyncTestCase):
         self.assertEqual(config.steps, 3000)
         self.assertEqual(config.epochs, 30)
         self.assertEqual(config.hidden, [12, 40, 15])
-        self.assertEqual(config.predict, "TARGET")
+        self.assertEqual(config.predict.NAME, "TARGET")
 
     async def test_00_train(self):
         async with self.sources as sources, self.model as model:
@@ -112,7 +113,7 @@ class TestDNN(AsyncTestCase):
         async with self.sources as sources, self.model as model:
             async with sources() as sctx, model() as mctx:
                 res = await mctx.accuracy(sctx)
-                self.assertGreater(res, 0.9)
+                self.assertGreater(res, 0.8)
 
     async def test_02_predict(self):
         test_feature_val = [
@@ -122,7 +123,7 @@ class TestDNN(AsyncTestCase):
         ]  # inserting zero so that its 1-indexable
         test_target = 2 * test_feature_val[1] + 3 * test_feature_val[2]
         # should be same function used in TestDNN.setupclass
-        a = Repo(
+        a = Record(
             "a",
             data={
                 "features": {
@@ -132,14 +133,17 @@ class TestDNN(AsyncTestCase):
             },
         )
         async with Sources(
-            MemorySource(MemorySourceConfig(repos=[a]))
+            MemorySource(MemorySourceConfig(records=[a]))
         ) as sources, self.model as model:
+            target_name = model.config.predict.NAME
             async with sources() as sctx, model() as mctx:
-                res = [repo async for repo in mctx.predict(sctx.repos())]
+                res = [record async for record in mctx.predict(sctx.records())]
                 self.assertEqual(len(res), 1)
-            self.assertEqual(res[0].src_url, a.src_url)
+            self.assertEqual(res[0].key, a.key)
             test_error_norm = abs(
-                (test_target - res[0].prediction().value) / test_target + 1e-6
+                (test_target - res[0].prediction(target_name).value)
+                / test_target
+                + 1e-6
             )
-            error_threshold = 0.2
+            error_threshold = 0.3
             self.assertLess(test_error_norm, error_threshold)
